@@ -12,7 +12,7 @@
 //===----------------------------------------------------------------------===//
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 #include "MCRegisterInfo.h"
 
@@ -22,18 +22,18 @@
 /// defined below.
 typedef struct DiffListIterator {
 	uint16_t Val;
-	MCPhysReg *List;
+	const MCPhysReg *List;
 } DiffListIterator;
 
 void MCRegisterInfo_InitMCRegisterInfo(MCRegisterInfo *RI,
-		MCRegisterDesc *D, unsigned NR,
+		const MCRegisterDesc *D, unsigned NR,
 		unsigned RA, unsigned PC,
-		MCRegisterClass *C, unsigned NC,
+		const MCRegisterClass *C, unsigned NC,
 		uint16_t (*RURoots)[2], unsigned NRU,
-		MCPhysReg *DL,
-		char *Strings,
-		uint16_t *SubIndices, unsigned NumIndices,
-		uint16_t *RET)
+		const MCPhysReg *DL,
+		const char *Strings,
+		const uint16_t *SubIndices, unsigned NumIndices,
+		const uint16_t *RET)
 {
 	RI->Desc = D;
 	RI->NumRegs = NR;
@@ -50,7 +50,7 @@ void MCRegisterInfo_InitMCRegisterInfo(MCRegisterInfo *RI,
 	RI->RegEncodingTable = RET;
 }
 
-static void DiffListIterator_init(DiffListIterator *d, MCPhysReg InitVal, MCPhysReg *DiffList)
+static void DiffListIterator_init(DiffListIterator *d, MCPhysReg InitVal, const MCPhysReg *DiffList)
 {
 	d->Val = InitVal;
 	d->List = DiffList;
@@ -83,7 +83,7 @@ static bool DiffListIterator_isValid(DiffListIterator *d)
 	return (d->List != 0);
 }
 
-unsigned MCRegisterInfo_getMatchingSuperReg(MCRegisterInfo *RI, unsigned Reg, unsigned SubIdx, MCRegisterClass *RC)
+unsigned MCRegisterInfo_getMatchingSuperReg(const MCRegisterInfo *RI, unsigned Reg, unsigned SubIdx, const MCRegisterClass *RC)
 {
 	DiffListIterator iter;
 
@@ -105,10 +105,10 @@ unsigned MCRegisterInfo_getMatchingSuperReg(MCRegisterInfo *RI, unsigned Reg, un
 	return 0;
 }
 
-unsigned MCRegisterInfo_getSubReg(MCRegisterInfo *RI, unsigned Reg, unsigned Idx)
+unsigned MCRegisterInfo_getSubReg(const MCRegisterInfo *RI, unsigned Reg, unsigned Idx)
 {
 	DiffListIterator iter;
-	uint16_t *SRI = RI->SubRegIndices + RI->Desc[Reg].SubRegIndices;
+	const uint16_t *SRI = RI->SubRegIndices + RI->Desc[Reg].SubRegIndices;
 
 	DiffListIterator_init(&iter, (MCPhysReg)Reg, RI->DiffLists + RI->Desc[Reg].SubRegs);
 	DiffListIterator_next(&iter);
@@ -123,7 +123,7 @@ unsigned MCRegisterInfo_getSubReg(MCRegisterInfo *RI, unsigned Reg, unsigned Idx
 	return 0;
 }
 
-MCRegisterClass* MCRegisterInfo_getRegClass(MCRegisterInfo *RI, unsigned i)
+const MCRegisterClass* MCRegisterInfo_getRegClass(const MCRegisterInfo *RI, unsigned i)
 {
 	//assert(i < getNumRegClasses() && "Register Class ID out of range");
 	if (i >= RI->NumClasses)
@@ -131,7 +131,7 @@ MCRegisterClass* MCRegisterInfo_getRegClass(MCRegisterInfo *RI, unsigned i)
 	return &(RI->Classes[i]);
 }
 
-bool MCRegisterClass_contains(MCRegisterClass *c, unsigned Reg)
+bool MCRegisterClass_contains(const MCRegisterClass *c, unsigned Reg)
 {
 	unsigned InByte = Reg % 8;
 	unsigned Byte = Reg / 8;
