@@ -12,15 +12,15 @@
 //===----------------------------------------------------------------------===//
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 #ifndef CS_LLVM_SUPPORT_MATHEXTRAS_H
 #define CS_LLVM_SUPPORT_MATHEXTRAS_H
 
-#include <stdint.h>
-
-#ifdef _MSC_VER
-# include <intrin.h>
+#if defined(_WIN32_WCE) && (_WIN32_WCE < 0x800)
+#include "windowsce/intrin.h"
+#elif defined(_MSC_VER)
+#include <intrin.h>
 #endif
 
 #ifndef __cplusplus
@@ -423,14 +423,17 @@ static inline int64_t SignExtend64(uint64_t X, unsigned B) {
 ///   valid arguments.
 static inline unsigned int countLeadingZeros(int x)
 {
-	unsigned count = 0;
 	int i;
 	const unsigned bits = sizeof(x) * 8;
+	unsigned count = bits;
 
+	if (x < 0) {
+		return 0;
+	}
 	for (i = bits; --i; ) {
-		if (x < 0) break;
-		count++;
-		x <<= 1;
+		if (x == 0) break;
+		count--;
+		x >>= 1;
 	}
 
 	return count;
