@@ -1,9 +1,11 @@
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 #ifndef CS_SSTREAM_H_
 #define CS_SSTREAM_H_
 
+#include "include/capstone/platform.h"
+
 typedef struct SStream {
 	char buffer[512];
 	int index;
@@ -13,7 +15,7 @@ void SStream_Init(SStream *ss);
 
 void SStream_concat(SStream *ss, const char *fmt, ...);
 
-void SStream_concat0(SStream *ss, char *s);
+void SStream_concat0(SStream *ss, const char *s);
 
 void printInt64Bang(SStream *O, int64_t val);
 
