@@ -15,12 +15,12 @@
 //===----------------------------------------------------------------------===//
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 #ifndef CS_ARMBASEINFO_H
 #define CS_ARMBASEINFO_H
 
-#include "../../include/arm.h"
+#include "capstone/arm.h"
 
 // Defines symbolic names for ARM registers.  This defines a mapping from
 // register name to register number.
@@ -70,7 +70,7 @@ inline static ARMCC_CondCodes ARMCC_getOppositeCondition(ARMCC_CondCodes CC)
 	}
 }
 
-inline static char *ARMCC_ARMCondCodeToString(ARMCC_CondCodes CC)
+inline static const char *ARMCC_ARMCondCodeToString(ARMCC_CondCodes CC)
 {
 	switch (CC) {
 		case ARMCC_EQ:  return "eq";
@@ -92,7 +92,7 @@ inline static char *ARMCC_ARMCondCodeToString(ARMCC_CondCodes CC)
 	}
 }
 
-inline static char *ARM_PROC_IFlagsToString(unsigned val)
+inline static const char *ARM_PROC_IFlagsToString(unsigned val)
 {
 	switch (val) {
 		case ARM_CPSFLAG_F: return "f";
@@ -102,7 +102,7 @@ inline static char *ARM_PROC_IFlagsToString(unsigned val)
 	}
 }
 
-inline static char *ARM_PROC_IModToString(unsigned val)
+inline static const char *ARM_PROC_IModToString(unsigned val)
 {
 	switch (val) {
 		case ARM_CPSMODE_IE: return "ie";
@@ -111,7 +111,7 @@ inline static char *ARM_PROC_IModToString(unsigned val)
 	}
 }
 
-inline static char *ARM_MB_MemBOptToString(unsigned val, bool HasV8)
+inline static const char *ARM_MB_MemBOptToString(unsigned val, bool HasV8)
 {
 	switch (val) {
 		default: return "BUGBUG";
@@ -153,7 +153,7 @@ enum ARM_ISB_InstSyncBOpt {
     ARM_ISB_SY = 15
 };
 
-inline static char *ARM_ISB_InstSyncBOptToString(unsigned val)
+inline static const char *ARM_ISB_InstSyncBOptToString(unsigned val)
 {
 	switch (val) {
 		default: // never reach
@@ -222,7 +222,7 @@ typedef enum ARMII_AddrMode {
 	ARMII_AddrMode_i12    = 16
 } ARMII_AddrMode;
 
-inline static char *ARMII_AddrModeToString(ARMII_AddrMode addrmode)
+inline static const char *ARMII_AddrModeToString(ARMII_AddrMode addrmode)
 {
 	switch (addrmode) {
 		case ARMII_AddrModeNone:    return "AddrModeNone";
