@@ -12,7 +12,7 @@
 //===----------------------------------------------------------------------===//
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 #ifdef CAPSTONE_HAS_POWERPC
 
@@ -30,17 +30,33 @@
 #include "PPCMapping.h"
 
 #ifndef CAPSTONE_DIET
-static char *getRegisterName(unsigned RegNo);
+static const char *getRegisterName(unsigned RegNo);
 #endif
 
 static void printOperand(MCInst *MI, unsigned OpNo, SStream *O);
-static void printInstruction(MCInst *MI, SStream *O, MCRegisterInfo *MRI);
+static void printInstruction(MCInst *MI, SStream *O, const MCRegisterInfo *MRI);
 static void printAbsBranchOperand(MCInst *MI, unsigned OpNo, SStream *O);
 static char *printAliasInstr(MCInst *MI, SStream *OS, void *info);
 static char *printAliasInstrEx(MCInst *MI, SStream *OS, void *info);
 static void printCustomAliasOperand(MCInst *MI, unsigned OpIdx,
 		unsigned PrintMethodIdx, SStream *OS);
 
+#if 0
+static void printRegName(SStream *OS, unsigned RegNo)
+{
+	char *RegName = getRegisterName(RegNo);
+
+	if (RegName[0] == 'q' /* QPX */) {
+		// The system toolchain on the BG/Q does not understand QPX register names
+		// in .cfi_* directives, so print the name of the floating-point
+		// subregister instead.
+		RegName[0] = 'f';
+	}
+
+	SStream_concat0(OS, RegName);
+}
+#endif
+
 static void set_mem_access(MCInst *MI, bool status)
 {
 	if (MI->csh->detail != CS_OPT_ON)
@@ -161,11 +177,10 @@ void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 	}
 
 	if (isBOCTRBranch(MCInst_getOpcode(MI))) {
-		if (MCOperand_isImm(MCInst_getOperand(MI,0)))
-		{
+		if (MCOperand_isImm(MCInst_getOperand(MI,0))) {
 			int64_t bd = MCOperand_getImm(MCInst_getOperand(MI, 0));
 			bd = SignExtend64(bd, 14);
-			MCOperand_setImm(MCInst_getOperand(MI, 0),bd);
+			MCOperand_setImm(MCInst_getOperand(MI, 0), bd);
 		}
 	}
 
@@ -183,14 +198,16 @@ void PPC_printInst(MCInst *MI, SStream *O, void *Info)
 
 	if (mnem != NULL) {
 		if (strlen(mnem) > 0) {
-			struct ppc_alias alias;
 			// check to remove the last letter of ('.', '-', '+')
 			if (mnem[strlen(mnem) - 1] == '-' || mnem[strlen(mnem) - 1] == '+' || mnem[strlen(mnem) - 1] == '.')
 				mnem[strlen(mnem) - 1] = '\0';
 
-			if (PPC_alias_insn(mnem, &alias)) {
-				MCInst_setOpcodePub(MI, alias.id);
-				if (MI->csh->detail) {
+			MCInst_setOpcodePub(MI, PPC_map_insn(mnem));
+
+			if (MI->csh->detail) {
+				struct ppc_alias alias;
+
+				if (PPC_alias_insn(mnem, &alias)) {
 					MI->flat_insn->detail->ppc.bc = (ppc_bc)alias.cc;
 				}
 			}
@@ -388,17 +405,7 @@ static void printS5ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	int Value = (int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 	Value = SignExtend32(Value, 5);
 
-	if (Value >= 0) {
-		if (Value > HEX_THRESHOLD)
-			SStream_concat(O, "0x%x", Value);
-		else
-			SStream_concat(O, "%u", Value);
-	} else {
-		if (Value < -HEX_THRESHOLD)
-			SStream_concat(O, "-0x%x", -Value);
-		else
-			SStream_concat(O, "-%u", -Value);
-	}
+	printInt32(O, Value);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -411,10 +418,7 @@ static void printU5ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 	//assert(Value <= 31 && "Invalid u5imm argument!");
-	if (Value > HEX_THRESHOLD)
-		SStream_concat(O, "0x%x", Value);
-	else
-		SStream_concat(O, "%u", Value);
+	printUInt32(O, Value);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -427,6 +431,21 @@ static void printU6ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	unsigned int Value = (unsigned int)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
 	//assert(Value <= 63 && "Invalid u6imm argument!");
+	printUInt32(O, Value);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
+		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = Value;
+		MI->flat_insn->detail->ppc.op_count++;
+	}
+}
+
+static void printU12ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	unsigned short Value = (unsigned short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+
+	// assert(Value <= 4095 && "Invalid u12imm argument!");
+
 	if (Value > HEX_THRESHOLD)
 		SStream_concat(O, "0x%x", Value);
 	else
@@ -442,18 +461,11 @@ static void printU6ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 static void printS16ImmOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
-		short Imm = (short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
-		if (Imm >= 0) {
-			if (Imm > HEX_THRESHOLD)
-				SStream_concat(O, "0x%x", Imm);
-			else
-				SStream_concat(O, "%u", Imm);
-		} else {
-			if (Imm < -HEX_THRESHOLD)
-				SStream_concat(O, "-0x%x", -Imm);
-			else
-				SStream_concat(O, "-%u", -Imm);
-		}
+		unsigned short Imm = (unsigned short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
+        if (Imm > HEX_THRESHOLD)
+            SStream_concat(O, "0x%x", Imm);
+        else
+            SStream_concat(O, "%u", Imm);
 
 		if (MI->csh->detail) {
 			MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -468,9 +480,6 @@ static void printS16ImmOperand_Mem(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	if (MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		short Imm = (short)MCOperand_getImm(MCInst_getOperand(MI, OpNo));
-		// Do not print zero offset
-		if (Imm == 0)
-			return;
 
 		if (Imm >= 0) {
 			if (Imm > HEX_THRESHOLD)
@@ -529,20 +538,20 @@ static void printBranchOperand(MCInst *MI, unsigned OpNo, SStream *O)
 
 static void printAbsBranchOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
-	int imm;
+	int64_t imm;
 
 	if (!MCOperand_isImm(MCInst_getOperand(MI, OpNo))) {
 		printOperand(MI, OpNo, O);
 		return;
 	}
 
-	imm = ((int)MCOperand_getImm(MCInst_getOperand(MI, OpNo)) << 2);
+	imm = MCOperand_getImm(MCInst_getOperand(MI, OpNo)) * 4;
 
 	if (!PPC_abs_branch(MI->csh, MCInst_getOpcode(MI))) {
-		imm = (int)MI->address + imm;
+		imm = MI->address + imm;
 	}
 
-	SStream_concat(O, "0x%x", imm);
+	SStream_concat(O, "0x%"PRIx64, imm);
 
 	if (MI->csh->detail) {
 		MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
@@ -627,11 +636,12 @@ static void printTLSCall(MCInst *MI, unsigned OpNo, SStream *O)
 #ifndef CAPSTONE_DIET
 /// stripRegisterPrefix - This method strips the character prefix from a
 /// register name so that only the number is left.  Used by for linux asm.
-static char *stripRegisterPrefix(char *RegName)
+static const char *stripRegisterPrefix(const char *RegName)
 {
 	switch (RegName[0]) {
 		case 'r':
 		case 'f':
+		case 'q': // for QPX
 		case 'v':
 			if (RegName[1] == 's')
 				return RegName + 2;
@@ -651,7 +661,7 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	if (MCOperand_isReg(Op)) {
 		unsigned reg = MCOperand_getReg(Op);
 #ifndef CAPSTONE_DIET
-		char *RegName = getRegisterName(reg);
+		const char *RegName = getRegisterName(reg);
 #endif
 		// map to public register
 		reg = PPC_map_register(reg);
@@ -678,21 +688,11 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 
 	if (MCOperand_isImm(Op)) {
 		int32_t imm = (int32_t)MCOperand_getImm(Op);
-		if (imm >= 0) {
-			if (imm > HEX_THRESHOLD)
-				SStream_concat(O, "0x%x", imm);
-			else
-				SStream_concat(O, "%u", imm);
-		} else {
-			if (imm < -HEX_THRESHOLD)
-				SStream_concat(O, "-0x%x", -imm);
-			else
-				SStream_concat(O, "-%u", -imm);
-		}
+		printInt32(O, imm);
 
 		if (MI->csh->detail) {
 			if (MI->csh->doing_mem) {
-				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].mem.disp = imm;
+				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].mem.disp = (int32_t)imm;
 			} else {
 				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].type = PPC_OP_IMM;
 				MI->flat_insn->detail->ppc.operands[MI->flat_insn->detail->ppc.op_count].imm = imm;
