@@ -13,16 +13,23 @@
 //===----------------------------------------------------------------------===//
 
 /* Capstone Disassembly Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2015 */
 
 // this code is only relevant when DIET mode is disable
 #if defined(CAPSTONE_HAS_X86) && !defined(CAPSTONE_DIET) && !defined(CAPSTONE_X86_ATT_DISABLE)
 
+#if defined (WIN32) || defined (WIN64) || defined (_WIN32) || defined (_WIN64)
+#pragma warning(disable:4996)			// disable MSVC's warning on strncpy()
+#pragma warning(disable:28719)		// disable MSVC's warning on strncpy()
+#endif
+
 #if !defined(CAPSTONE_HAS_OSXKERNEL)
 #include <ctype.h>
 #endif
-#include "../../myinttypes.h"
+#include <capstone/platform.h>
+
 #if defined(CAPSTONE_HAS_OSXKERNEL)
+#include <Availability.h>
 #include <libkern/libkern.h>
 #else
 #include <stdio.h>
@@ -65,13 +72,51 @@ static void printopaquemem(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	switch(MI->csh->mode) {
 		case CS_MODE_16:
-			MI->x86opsize = 2;
+			switch(MI->flat_insn->id) {
+				default:
+					MI->x86opsize = 2;
+					break;
+				case X86_INS_LJMP:
+				case X86_INS_LCALL:
+					MI->x86opsize = 4;
+					break;
+				case X86_INS_SGDT:
+				case X86_INS_SIDT:
+				case X86_INS_LGDT:
+				case X86_INS_LIDT:
+					MI->x86opsize = 6;
+					break;
+			}
 			break;
 		case CS_MODE_32:
-			MI->x86opsize = 4;
+			switch(MI->flat_insn->id) {
+				default:
+					MI->x86opsize = 4;
+					break;
+				case X86_INS_LJMP:
+				case X86_INS_LCALL:
+				case X86_INS_SGDT:
+				case X86_INS_SIDT:
+				case X86_INS_LGDT:
+				case X86_INS_LIDT:
+					MI->x86opsize = 6;
+					break;
+			}
 			break;
 		case CS_MODE_64:
-			MI->x86opsize = 8;
+			switch(MI->flat_insn->id) {
+				default:
+					MI->x86opsize = 8;
+					break;
+				case X86_INS_LJMP:
+				case X86_INS_LCALL:
+				case X86_INS_SGDT:
+				case X86_INS_SIDT:
+				case X86_INS_LGDT:
+				case X86_INS_LIDT:
+					MI->x86opsize = 10;
+					break;
+			}
 			break;
 		default:	// never reach
 			break;
@@ -127,7 +172,32 @@ static void printi512mem(MCInst *MI, unsigned OpNo, SStream *O)
 
 static void printf32mem(MCInst *MI, unsigned OpNo, SStream *O)
 {
-	MI->x86opsize = 4;
+	switch(MCInst_getOpcode(MI)) {
+		default:
+			MI->x86opsize = 4;
+			break;
+		case X86_FBSTPm:
+		case X86_FBLDm:
+			// TODO: fix this in tablegen instead
+			MI->x86opsize = 10;
+			break;
+		case X86_FSTENVm:
+		case X86_FLDENVm:
+			// TODO: fix this in tablegen instead
+			switch(MI->csh->mode) {
+				default:    // never reach
+					break;
+				case CS_MODE_16:
+					MI->x86opsize = 14;
+					break;
+				case CS_MODE_32:
+				case CS_MODE_64:
+					MI->x86opsize = 28;
+					break;
+			}
+			break;
+	}
+
 	printMemReference(MI, OpNo, O);
 }
 
@@ -163,7 +233,7 @@ static void printf512mem(MCInst *MI, unsigned OpNo, SStream *O)
 
 static void printSSECC(MCInst *MI, unsigned Op, SStream *OS)
 {
-	int64_t Imm = MCOperand_getImm(MCInst_getOperand(MI, Op)) & 7;
+	uint8_t Imm = (uint8_t)(MCOperand_getImm(MCInst_getOperand(MI, Op)) & 7);
 	switch (Imm) {
 		default: break;	// never reach
 		case    0: SStream_concat0(OS, "eq"); op_addSseCC(MI, X86_SSE_CC_EQ); break;
@@ -174,20 +244,14 @@ static void printSSECC(MCInst *MI, unsigned Op, SStream *OS)
 		case    5: SStream_concat0(OS, "nlt"); op_addSseCC(MI, X86_SSE_CC_NLT); break;
 		case    6: SStream_concat0(OS, "nle"); op_addSseCC(MI, X86_SSE_CC_NLE); break;
 		case    7: SStream_concat0(OS, "ord"); op_addSseCC(MI, X86_SSE_CC_ORD); break;
-		case    8: SStream_concat0(OS, "eq_uq"); op_addSseCC(MI, X86_SSE_CC_EQ_UQ); break;
-		case    9: SStream_concat0(OS, "nge"); op_addSseCC(MI, X86_SSE_CC_NGE); break;
-		case  0xa: SStream_concat0(OS, "ngt"); op_addSseCC(MI, X86_SSE_CC_NGT); break;
-		case  0xb: SStream_concat0(OS, "false"); op_addSseCC(MI, X86_SSE_CC_FALSE); break;
-		case  0xc: SStream_concat0(OS, "neq_oq"); op_addSseCC(MI, X86_SSE_CC_NEQ_OQ); break;
-		case  0xd: SStream_concat0(OS, "ge"); op_addSseCC(MI, X86_SSE_CC_GE); break;
-		case  0xe: SStream_concat0(OS, "gt"); op_addSseCC(MI, X86_SSE_CC_GT); break;
-		case  0xf: SStream_concat0(OS, "true"); op_addSseCC(MI, X86_SSE_CC_TRUE); break;
 	}
+
+	MI->popcode_adjust = Imm + 1;
 }
 
 static void printAVXCC(MCInst *MI, unsigned Op, SStream *O)
 {
-	int64_t Imm = MCOperand_getImm(MCInst_getOperand(MI, Op)) & 0x1f;
+	uint8_t Imm = (uint8_t)(MCOperand_getImm(MCInst_getOperand(MI, Op)) & 0x1f);
 	switch (Imm) {
 		default: break;//printf("Invalid avxcc argument!\n"); break;
 		case    0: SStream_concat0(O, "eq"); op_addAvxCC(MI, X86_AVX_CC_EQ); break;
@@ -223,6 +287,25 @@ static void printAVXCC(MCInst *MI, unsigned Op, SStream *O)
 		case 0x1e: SStream_concat0(O, "gt_oq"); op_addAvxCC(MI, X86_AVX_CC_GT_OQ); break;
 		case 0x1f: SStream_concat0(O, "true_us"); op_addAvxCC(MI, X86_AVX_CC_TRUE_US); break;
 	}
+
+	MI->popcode_adjust = Imm + 1;
+}
+
+static void printXOPCC(MCInst *MI, unsigned Op, SStream *O)
+{
+	int64_t Imm = MCOperand_getImm(MCInst_getOperand(MI, Op));
+
+	switch (Imm) {
+		default: // llvm_unreachable("Invalid xopcc argument!");
+		case 0: SStream_concat0(O, "lt"); op_addXopCC(MI, X86_XOP_CC_LT); break;
+		case 1: SStream_concat0(O, "le"); op_addXopCC(MI, X86_XOP_CC_LE); break;
+		case 2: SStream_concat0(O, "gt"); op_addXopCC(MI, X86_XOP_CC_GT); break;
+		case 3: SStream_concat0(O, "ge"); op_addXopCC(MI, X86_XOP_CC_GE); break;
+		case 4: SStream_concat0(O, "eq"); op_addXopCC(MI, X86_XOP_CC_EQ); break;
+		case 5: SStream_concat0(O, "neq"); op_addXopCC(MI, X86_XOP_CC_NEQ); break;
+		case 6: SStream_concat0(O, "false"); op_addXopCC(MI, X86_XOP_CC_FALSE); break;
+		case 7: SStream_concat0(O, "true"); op_addXopCC(MI, X86_XOP_CC_TRUE); break;
+	}
 }
 
 static void printRoundingControl(MCInst *MI, unsigned Op, SStream *O)
@@ -250,11 +333,34 @@ static void _printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	} else if (MCOperand_isImm(Op)) {
 		// Print X86 immediates as signed values.
 		int64_t imm = MCOperand_getImm(Op);
+		uint8_t encsize;
+		uint8_t opsize = X86_immediate_size(MCInst_getOpcode(MI), &encsize);
+
 		if (imm < 0) {
-			if (imm < -HEX_THRESHOLD)
-				SStream_concat(O, "$-0x%"PRIx64, -imm);
-			else
-				SStream_concat(O, "$-%"PRIu64, -imm);
+			if (MI->csh->imm_unsigned) {
+				if (opsize) {
+					switch(opsize) {
+						default:
+							break;
+						case 1:
+							imm &= 0xff;
+							break;
+						case 2:
+							imm &= 0xffff;
+							break;
+						case 4:
+							imm &= 0xffffffff;
+							break;
+					}
+				}
+
+				SStream_concat(O, "$0x%"PRIx64, imm);
+			} else {
+				if (imm < -HEX_THRESHOLD)
+					SStream_concat(O, "$-0x%"PRIx64, -imm);
+				else
+					SStream_concat(O, "$-%"PRIu64, -imm);
+			}
 		} else {
 			if (imm > HEX_THRESHOLD)
 				SStream_concat(O, "$0x%"PRIx64, imm);
@@ -264,12 +370,41 @@ static void _printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 	}
 }
 
+// convert Intel access info to AT&T access info
+static void get_op_access(cs_struct *h, unsigned int id, uint8_t *access, uint64_t *eflags)
+{
+	uint8_t count, i;
+	uint8_t *arr = X86_get_op_access(h, id, eflags);
+
+	if (!arr) {
+		access[0] = 0;
+		return;
+	}
+
+	// find the non-zero last entry
+	for(count = 0; arr[count]; count++);
+
+	if (count == 0)
+		return;
+
+	// copy in reverse order this access array from Intel syntax -> AT&T syntax
+	count--;
+	for(i = 0; i <= count; i++) {
+		if (arr[count - i] != CS_AC_IGNORE)
+			access[i] = arr[count - i];
+		else
+			access[i] = 0;
+	}
+}
+
 static void printSrcIdx(MCInst *MI, unsigned Op, SStream *O)
 {
 	MCOperand *SegReg;
 	int reg;
 
 	if (MI->csh->detail) {
+		uint8_t access[6];
+
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_MEM;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->x86opsize;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = X86_REG_INVALID;
@@ -277,6 +412,9 @@ static void printSrcIdx(MCInst *MI, unsigned Op, SStream *O)
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.index = X86_REG_INVALID;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.scale = 1;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = 0;
+
+		get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].access = access[MI->flat_insn->detail->x86.op_count];
 	}
 
 	SegReg = MCInst_getOperand(MI, Op+1);
@@ -304,6 +442,8 @@ static void printSrcIdx(MCInst *MI, unsigned Op, SStream *O)
 static void printDstIdx(MCInst *MI, unsigned Op, SStream *O)
 {
 	if (MI->csh->detail) {
+		uint8_t access[6];
+
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_MEM;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->x86opsize;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = X86_REG_INVALID;
@@ -311,6 +451,9 @@ static void printDstIdx(MCInst *MI, unsigned Op, SStream *O)
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.index = X86_REG_INVALID;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.scale = 1;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = 0;
+
+		get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].access = access[MI->flat_insn->detail->x86.op_count];
 	}
 
 	// DI accesses are always ES-based on non-64bit mode
@@ -385,6 +528,8 @@ static void printMemOffset(MCInst *MI, unsigned Op, SStream *O)
 	int reg;
 
 	if (MI->csh->detail) {
+		uint8_t access[6];
+
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_MEM;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->x86opsize;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = X86_REG_INVALID;
@@ -392,6 +537,9 @@ static void printMemOffset(MCInst *MI, unsigned Op, SStream *O)
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.index = X86_REG_INVALID;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.scale = 1;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = 0;
+
+		get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].access = access[MI->flat_insn->detail->x86.op_count];
 	}
 
 	// If this has a segment register, print it.
@@ -422,6 +570,25 @@ static void printMemOffset(MCInst *MI, unsigned Op, SStream *O)
 		MI->flat_insn->detail->x86.op_count++;
 }
 
+#ifndef CAPSTONE_X86_REDUCE
+static void printU8Imm(MCInst *MI, unsigned Op, SStream *O)
+{
+	uint8_t val = MCOperand_getImm(MCInst_getOperand(MI, Op)) & 0xff;
+
+	if (val > HEX_THRESHOLD)
+		SStream_concat(O, "$0x%x", val);
+	else
+		SStream_concat(O, "$%u", val);
+
+	if (MI->csh->detail) {
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_IMM;
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].imm = val;
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = 1;
+		MI->flat_insn->detail->x86.op_count++;
+	}
+}
+#endif
+
 static void printMemOffs8(MCInst *MI, unsigned OpNo, SStream *O)
 {
 	MI->x86opsize = 1;
@@ -461,13 +628,22 @@ static void printPCRelImm(MCInst *MI, unsigned OpNo, SStream *O)
 			imm = imm & 0xffffffff;
 		}
 
+		if (MI->csh->mode == CS_MODE_16 &&
+				(MI->Opcode != X86_JMP_4 && MI->Opcode != X86_CALLpcrel32))
+			imm = imm & 0xffff;
+
+		// Hack: X86 16bit with opcode X86_JMP_4
+		if (MI->csh->mode == CS_MODE_16 &&
+				(MI->Opcode == X86_JMP_4 && MI->x86_prefix[2] != 0x66))
+			imm = imm & 0xffff;
+
+		// CALL/JMP rel16 is special
+		if (MI->Opcode == X86_CALLpcrel16 || MI->Opcode == X86_JMP_2)
+			imm = imm & 0xffff;
+
 		if (imm < 0) {
 			SStream_concat(O, "0x%"PRIx64, imm);
 		} else {
-			// handle 16bit segment bound
-			if (MI->csh->mode == CS_MODE_16 && imm > 0x100000)
-				imm -= 0x10000;
-
 			if (imm > HEX_THRESHOLD)
 				SStream_concat(O, "0x%"PRIx64, imm);
 			else
@@ -484,9 +660,7 @@ static void printPCRelImm(MCInst *MI, unsigned OpNo, SStream *O)
 
 static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
-	int opsize = 0;
 	MCOperand *Op  = MCInst_getOperand(MI, OpNo);
-
 	if (MCOperand_isReg(Op)) {
 		unsigned int reg = MCOperand_getReg(Op);
 		printRegName(O, reg);
@@ -494,91 +668,26 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 			if (MI->csh->doing_mem) {
 				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.base = reg;
 			} else {
+				uint8_t access[6];
+
 				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_REG;
 				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].reg = reg;
 				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->csh->regsize_map[reg];
+
+				get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].access = access[MI->flat_insn->detail->x86.op_count];
+
 				MI->flat_insn->detail->x86.op_count++;
 			}
 		}
 	} else if (MCOperand_isImm(Op)) {
 		// Print X86 immediates as signed values.
+		uint8_t encsize;
 		int64_t imm = MCOperand_getImm(Op);
+		uint8_t opsize = X86_immediate_size(MCInst_getOpcode(MI), &encsize);
 
-		switch(MCInst_getOpcode(MI)) {
-			default:
-				break;
-
-			case X86_AAD8i8:
-			case X86_AAM8i8:
-			case X86_ADC8i8:
-			case X86_ADD8i8:
-			case X86_AND8i8:
-			case X86_CMP8i8:
-			case X86_OR8i8:
-			case X86_SBB8i8:
-			case X86_SUB8i8:
-			case X86_TEST8i8:
-			case X86_XOR8i8:
-			case X86_ROL8ri:
-			case X86_ADC8ri:
-			case X86_ADD8ri:
-			case X86_ADD8ri8:
-			case X86_AND8ri:
-			case X86_AND8ri8:
-			case X86_CMP8ri:
-			case X86_MOV8ri:
-			case X86_MOV8ri_alt:
-			case X86_OR8ri:
-			case X86_OR8ri8:
-			case X86_RCL8ri:
-			case X86_RCR8ri:
-			case X86_ROR8ri:
-			case X86_SAL8ri:
-			case X86_SAR8ri:
-			case X86_SBB8ri:
-			case X86_SHL8ri:
-			case X86_SHR8ri:
-			case X86_SUB8ri:
-			case X86_SUB8ri8:
-			case X86_TEST8ri:
-			case X86_TEST8ri_NOREX:
-			case X86_TEST8ri_alt:
-			case X86_XOR8ri:
-			case X86_XOR8ri8:
-			case X86_OUT8ir:
-
-			case X86_ADC8mi:
-			case X86_ADD8mi:
-			case X86_AND8mi:
-			case X86_CMP8mi:
-			case X86_LOCK_ADD8mi:
-			case X86_LOCK_AND8mi:
-			case X86_LOCK_OR8mi:
-			case X86_LOCK_SUB8mi:
-			case X86_LOCK_XOR8mi:
-			case X86_MOV8mi:
-			case X86_OR8mi:
-			case X86_RCL8mi:
-			case X86_RCR8mi:
-			case X86_ROL8mi:
-			case X86_ROR8mi:
-			case X86_SAL8mi:
-			case X86_SAR8mi:
-			case X86_SBB8mi:
-			case X86_SHL8mi:
-			case X86_SHR8mi:
-			case X86_SUB8mi:
-			case X86_TEST8mi:
-			case X86_TEST8mi_alt:
-			case X86_XOR8mi:
-			case X86_PUSH64i8:
-			case X86_CMP32ri8:
-			case X86_CMP64ri8:
-
-				imm = imm & 0xff;
-				opsize = 1;     // immediate of 1 byte
-				break;
-		}
+		if (opsize == 1)    // print 1 byte immediate in positive form
+			imm = imm & 0xff;
 
 		switch(MI->flat_insn->id) {
 			default:
@@ -588,13 +697,62 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 					else
 						SStream_concat(O, "$%"PRIu64, imm);
 				} else {
-					if (imm < -HEX_THRESHOLD)
-						SStream_concat(O, "$-0x%"PRIx64, -imm);
-					else
-						SStream_concat(O, "$-%"PRIu64, -imm);
+					if (MI->csh->imm_unsigned) {
+						if (opsize) {
+							switch(opsize) {
+								default:
+									break;
+								case 1:
+									imm &= 0xff;
+									break;
+								case 2:
+									imm &= 0xffff;
+									break;
+								case 4:
+									imm &= 0xffffffff;
+									break;
+							}
+						}
+
+						SStream_concat(O, "$0x%"PRIx64, imm);
+					} else {
+						if (imm == 0x8000000000000000LL)  // imm == -imm
+							SStream_concat0(O, "$0x8000000000000000");
+						else if (imm < -HEX_THRESHOLD)
+							SStream_concat(O, "$-0x%"PRIx64, -imm);
+						else
+							SStream_concat(O, "$-%"PRIu64, -imm);
+					}
 				}
 				break;
 
+			case X86_INS_MOVABS:
+				// do not print number in negative form
+				SStream_concat(O, "$0x%"PRIx64, imm);
+				break;
+
+			case X86_INS_IN:
+			case X86_INS_OUT:
+			case X86_INS_INT:
+				// do not print number in negative form
+				imm = imm & 0xff;
+				if (imm >= 0 && imm <= HEX_THRESHOLD)
+					SStream_concat(O, "$%u", imm);
+				else {
+					SStream_concat(O, "$0x%x", imm);
+				}
+				break;
+
+			case X86_INS_LCALL:
+			case X86_INS_LJMP:
+				// always print address in positive form
+				if (OpNo == 1) {	// selector is ptr16
+					imm = imm & 0xffff;
+					opsize = 2;
+				}
+				SStream_concat(O, "$0x%"PRIx64, imm);
+				break;
+
 			case X86_INS_AND:
 			case X86_INS_OR:
 			case X86_INS_XOR:
@@ -602,12 +760,13 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 				if (imm >= 0 && imm <= HEX_THRESHOLD)
 					SStream_concat(O, "$%u", imm);
 				else {
-					imm = arch_masks[MI->op1_size? MI->op1_size : MI->imm_size] & imm;
+					imm = arch_masks[opsize? opsize : MI->imm_size] & imm;
 					SStream_concat(O, "$0x%"PRIx64, imm);
 				}
 				break;
 
 			case X86_INS_RET:
+			case X86_INS_RETF:
 				// RET imm16
 				if (imm >= 0 && imm <= HEX_THRESHOLD)
 					SStream_concat(O, "$%u", imm);
@@ -627,9 +786,10 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 				MI->has_imm = true;
 				MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].imm = imm;
 
-				if (opsize > 0)
+				if (opsize > 0) {
 					MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = opsize;
-				else if (MI->op1_size > 0)
+					MI->flat_insn->detail->x86.encoding.imm_size = encsize;
+				} else if (MI->op1_size > 0)
 					MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->op1_size;
 				else
 					MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->imm_size;
@@ -647,9 +807,12 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 	MCOperand *DispSpec = MCInst_getOperand(MI, Op + X86_AddrDisp);
 	MCOperand *SegReg = MCInst_getOperand(MI, Op + X86_AddrSegmentReg);
 	uint64_t ScaleVal;
-	int reg;
+	int segreg;
+	int64_t DispVal = 1;
 
 	if (MI->csh->detail) {
+		uint8_t access[6];
+
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].type = X86_OP_MEM;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].size = MI->x86opsize;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = X86_REG_INVALID;
@@ -657,36 +820,29 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.index = MCOperand_getReg(IndexReg);
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.scale = 1;
 		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = 0;
+
+		get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+		MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].access = access[MI->flat_insn->detail->x86.op_count];
 	}
 
 	// If this has a segment register, print it.
-	reg = MCOperand_getReg(SegReg);
-	if (reg) {
+	segreg = MCOperand_getReg(SegReg);
+	if (segreg) {
 		_printOperand(MI, Op + X86_AddrSegmentReg, O);
 		if (MI->csh->detail) {
-			MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = reg;
+			MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.segment = segreg;
 		}
 
 		SStream_concat0(O, ":");
 	}
 
 	if (MCOperand_isImm(DispSpec)) {
-		int64_t DispVal = MCOperand_getImm(DispSpec);
+		DispVal = MCOperand_getImm(DispSpec);
 		if (MI->csh->detail)
 			MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count].mem.disp = DispVal;
 		if (DispVal) {
 			if (MCOperand_getReg(IndexReg) || MCOperand_getReg(BaseReg)) {
-				if (DispVal < 0) {
-					if (DispVal <  -HEX_THRESHOLD)
-						SStream_concat(O, "-0x%"PRIx64, -DispVal);
-					else
-						SStream_concat(O, "-%"PRIu64, -DispVal);
-				} else {
-					if (DispVal > HEX_THRESHOLD)
-						SStream_concat(O, "0x%"PRIx64, DispVal);
-					else
-						SStream_concat(O, "%"PRIu64, DispVal);
-				}
+				printInt64(O, DispVal);
 			} else {
 				// only immediate as address of memory
 				if (DispVal < 0) {
@@ -698,6 +854,7 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 						SStream_concat(O, "%"PRIu64, DispVal);
 				}
 			}
+		} else {
 		}
 	}
 
@@ -718,12 +875,33 @@ static void printMemReference(MCInst *MI, unsigned Op, SStream *O)
 			}
 		}
 		SStream_concat0(O, ")");
+	} else {
+		if (!DispVal)
+			SStream_concat0(O, "0");
 	}
 
 	if (MI->csh->detail)
 		MI->flat_insn->detail->x86.op_count++;
 }
 
+static void printanymem(MCInst *MI, unsigned OpNo, SStream *O)
+{
+	switch(MI->Opcode) {
+		default: break;
+		case X86_LEA16r:
+				 MI->x86opsize = 2;
+				 break;
+		case X86_LEA32r:
+		case X86_LEA64_32r:
+				 MI->x86opsize = 4;
+				 break;
+		case X86_LEA64r:
+				 MI->x86opsize = 8;
+				 break;
+	}
+	printMemReference(MI, OpNo, O);
+}
+
 #include "X86InstPrinter.h"
 
 #define GET_REGINFO_ENUM
@@ -746,8 +924,15 @@ void X86_ATT_printInst(MCInst *MI, SStream *OS, void *info)
 {
 	char *mnem;
 	x86_reg reg, reg2;
+	enum cs_ac_type access1, access2;
 	int i;
 
+	// perhaps this instruction does not need printer
+	if (MI->assembly[0]) {
+		strncpy(OS->buffer, MI->assembly, sizeof(OS->buffer));
+		return;
+	}
+
 	// Output CALLpcrel32 as "callq" in 64-bit mode.
 	// In Intel annotation it's always emitted as "call".
 	//
@@ -768,99 +953,116 @@ void X86_ATT_printInst(MCInst *MI, SStream *OS, void *info)
 	else
 		printInstruction(MI, OS, info);
 
+	// HACK TODO: fix this in machine description
+	switch(MI->flat_insn->id) {
+		default: break;
+		case X86_INS_SYSEXIT:
+				 SStream_Init(OS);
+				 SStream_concat0(OS, "sysexit");
+				 break;
+	}
+
 	if (MI->has_imm) {
 		// if op_count > 1, then this operand's size is taken from the destination op
 		if (MI->flat_insn->detail->x86.op_count > 1) {
-			for (i = 0; i < MI->flat_insn->detail->x86.op_count; i++) {
-				if (MI->flat_insn->detail->x86.operands[i].type == X86_OP_IMM)
-					MI->flat_insn->detail->x86.operands[i].size = MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count - 1].size;
+			if (MI->flat_insn->id != X86_INS_LCALL && MI->flat_insn->id != X86_INS_LJMP) {
+				for (i = 0; i < MI->flat_insn->detail->x86.op_count; i++) {
+					if (MI->flat_insn->detail->x86.operands[i].type == X86_OP_IMM)
+						MI->flat_insn->detail->x86.operands[i].size =
+							MI->flat_insn->detail->x86.operands[MI->flat_insn->detail->x86.op_count - 1].size;
+				}
 			}
 		} else
 			MI->flat_insn->detail->x86.operands[0].size = MI->imm_size;
 	}
 
 	if (MI->csh->detail) {
-        // some instructions need to supply immediate 1 in the first op
-        switch(MCInst_getOpcode(MI)) {
-            default:
-                break;
-            case X86_SHL8r1:
-            case X86_SHL16r1:
-            case X86_SHL32r1:
-            case X86_SHL64r1:
-            case X86_SAL8r1:
-            case X86_SAL16r1:
-            case X86_SAL32r1:
-            case X86_SAL64r1:
-            case X86_SHR8r1:
-            case X86_SHR16r1:
-            case X86_SHR32r1:
-            case X86_SHR64r1:
-            case X86_SAR8r1:
-            case X86_SAR16r1:
-            case X86_SAR32r1:
-            case X86_SAR64r1:
-            case X86_RCL8r1:
-            case X86_RCL16r1:
-            case X86_RCL32r1:
-            case X86_RCL64r1:
-            case X86_RCR8r1:
-            case X86_RCR16r1:
-            case X86_RCR32r1:
-            case X86_RCR64r1:
-            case X86_ROL8r1:
-            case X86_ROL16r1:
-            case X86_ROL32r1:
-            case X86_ROL64r1:
-            case X86_ROR8r1:
-            case X86_ROR16r1:
-            case X86_ROR32r1:
-            case X86_ROR64r1:
-            case X86_SHL8m1:
-            case X86_SHL16m1:
-            case X86_SHL32m1:
-            case X86_SHL64m1:
-            case X86_SAL8m1:
-            case X86_SAL16m1:
-            case X86_SAL32m1:
-            case X86_SAL64m1:
-            case X86_SHR8m1:
-            case X86_SHR16m1:
-            case X86_SHR32m1:
-            case X86_SHR64m1:
-            case X86_SAR8m1:
-            case X86_SAR16m1:
-            case X86_SAR32m1:
-            case X86_SAR64m1:
-            case X86_RCL8m1:
-            case X86_RCL16m1:
-            case X86_RCL32m1:
-            case X86_RCL64m1:
-            case X86_RCR8m1:
-            case X86_RCR16m1:
-            case X86_RCR32m1:
-            case X86_RCR64m1:
-            case X86_ROL8m1:
-            case X86_ROL16m1:
-            case X86_ROL32m1:
-            case X86_ROL64m1:
-            case X86_ROR8m1:
-            case X86_ROR16m1:
-            case X86_ROR32m1:
-            case X86_ROR64m1:
-                // shift all the ops right to leave 1st slot for this new register op
-                memmove(&(MI->flat_insn->detail->x86.operands[1]), &(MI->flat_insn->detail->x86.operands[0]),
-                        sizeof(MI->flat_insn->detail->x86.operands[0]) * (ARR_SIZE(MI->flat_insn->detail->x86.operands) - 1));
-                MI->flat_insn->detail->x86.operands[0].type = X86_OP_IMM;
-                MI->flat_insn->detail->x86.operands[0].imm = 1;
-                MI->flat_insn->detail->x86.operands[0].size = 1;
-                MI->flat_insn->detail->x86.op_count++;
-        }
+		uint8_t access[6] = {0};
+
+		// some instructions need to supply immediate 1 in the first op
+		switch(MCInst_getOpcode(MI)) {
+			default:
+				break;
+			case X86_SHL8r1:
+			case X86_SHL16r1:
+			case X86_SHL32r1:
+			case X86_SHL64r1:
+			case X86_SAL8r1:
+			case X86_SAL16r1:
+			case X86_SAL32r1:
+			case X86_SAL64r1:
+			case X86_SHR8r1:
+			case X86_SHR16r1:
+			case X86_SHR32r1:
+			case X86_SHR64r1:
+			case X86_SAR8r1:
+			case X86_SAR16r1:
+			case X86_SAR32r1:
+			case X86_SAR64r1:
+			case X86_RCL8r1:
+			case X86_RCL16r1:
+			case X86_RCL32r1:
+			case X86_RCL64r1:
+			case X86_RCR8r1:
+			case X86_RCR16r1:
+			case X86_RCR32r1:
+			case X86_RCR64r1:
+			case X86_ROL8r1:
+			case X86_ROL16r1:
+			case X86_ROL32r1:
+			case X86_ROL64r1:
+			case X86_ROR8r1:
+			case X86_ROR16r1:
+			case X86_ROR32r1:
+			case X86_ROR64r1:
+			case X86_SHL8m1:
+			case X86_SHL16m1:
+			case X86_SHL32m1:
+			case X86_SHL64m1:
+			case X86_SAL8m1:
+			case X86_SAL16m1:
+			case X86_SAL32m1:
+			case X86_SAL64m1:
+			case X86_SHR8m1:
+			case X86_SHR16m1:
+			case X86_SHR32m1:
+			case X86_SHR64m1:
+			case X86_SAR8m1:
+			case X86_SAR16m1:
+			case X86_SAR32m1:
+			case X86_SAR64m1:
+			case X86_RCL8m1:
+			case X86_RCL16m1:
+			case X86_RCL32m1:
+			case X86_RCL64m1:
+			case X86_RCR8m1:
+			case X86_RCR16m1:
+			case X86_RCR32m1:
+			case X86_RCR64m1:
+			case X86_ROL8m1:
+			case X86_ROL16m1:
+			case X86_ROL32m1:
+			case X86_ROL64m1:
+			case X86_ROR8m1:
+			case X86_ROR16m1:
+			case X86_ROR32m1:
+			case X86_ROR64m1:
+				// shift all the ops right to leave 1st slot for this new register op
+				memmove(&(MI->flat_insn->detail->x86.operands[1]), &(MI->flat_insn->detail->x86.operands[0]),
+						sizeof(MI->flat_insn->detail->x86.operands[0]) * (ARR_SIZE(MI->flat_insn->detail->x86.operands) - 1));
+				MI->flat_insn->detail->x86.operands[0].type = X86_OP_IMM;
+				MI->flat_insn->detail->x86.operands[0].imm = 1;
+				MI->flat_insn->detail->x86.operands[0].size = 1;
+				MI->flat_insn->detail->x86.op_count++;
+		}
 
 		// special instruction needs to supply register op
 		// first op can be embedded in the asm by llvm.
 		// so we have to add the missing register as the first operand
-		reg = X86_insn_reg_att(MCInst_getOpcode(MI));
+
+		//printf(">>> opcode = %u\n", MCInst_getOpcode(MI));
+
+		reg = X86_insn_reg_att(MCInst_getOpcode(MI), &access1);
 		if (reg) {
 			// shift all the ops right to leave 1st slot for this new register op
 			memmove(&(MI->flat_insn->detail->x86.operands[1]), &(MI->flat_insn->detail->x86.operands[0]),
@@ -868,18 +1070,29 @@ void X86_ATT_printInst(MCInst *MI, SStream *OS, void *info)
 			MI->flat_insn->detail->x86.operands[0].type = X86_OP_REG;
 			MI->flat_insn->detail->x86.operands[0].reg = reg;
 			MI->flat_insn->detail->x86.operands[0].size = MI->csh->regsize_map[reg];
+			MI->flat_insn->detail->x86.operands[0].access = access1;
+
 			MI->flat_insn->detail->x86.op_count++;
 		} else {
-			if (X86_insn_reg_att2(MCInst_getOpcode(MI), &reg, &reg2)) {
+			if (X86_insn_reg_att2(MCInst_getOpcode(MI), &reg, &access1, &reg2, &access2)) {
+
 				MI->flat_insn->detail->x86.operands[0].type = X86_OP_REG;
 				MI->flat_insn->detail->x86.operands[0].reg = reg;
 				MI->flat_insn->detail->x86.operands[0].size = MI->csh->regsize_map[reg];
+				MI->flat_insn->detail->x86.operands[0].access = access1;
 				MI->flat_insn->detail->x86.operands[1].type = X86_OP_REG;
 				MI->flat_insn->detail->x86.operands[1].reg = reg2;
 				MI->flat_insn->detail->x86.operands[1].size = MI->csh->regsize_map[reg2];
+				MI->flat_insn->detail->x86.operands[0].access = access2;
 				MI->flat_insn->detail->x86.op_count = 2;
 			}
 		}
+
+#ifndef CAPSTONE_DIET
+		get_op_access(MI->csh, MCInst_getOpcode(MI), access, &MI->flat_insn->detail->x86.eflags);
+		MI->flat_insn->detail->x86.operands[0].access = access[0];
+		MI->flat_insn->detail->x86.operands[1].access = access[1];
+#endif
 	}
 }
 
