@@ -1,6 +1,7 @@
 # Capstone Python bindings, by Nguyen Anh Quynnh <aquynh@gmail.com>
 
-import ctypes, copy
+import ctypes
+from . import copy_ctypes_list
 from .ppc_const import *
 
 # define the API
@@ -20,7 +21,7 @@ class PpcOpCrx(ctypes.Structure):
 class PpcOpValue(ctypes.Union):
     _fields_ = (
         ('reg', ctypes.c_uint),
-        ('imm', ctypes.c_int32),
+        ('imm', ctypes.c_int64),
         ('mem', PpcOpMem),
         ('crx', PpcOpCrx),
     )
@@ -58,5 +59,5 @@ class CsPpc(ctypes.Structure):
     )
 
 def get_arch_info(a):
-    return (a.bc, a.bh, a.update_cr0, copy.deepcopy(a.operands[:a.op_count]))
+    return (a.bc, a.bh, a.update_cr0, copy_ctypes_list(a.operands[:a.op_count]))
 
