@@ -1,5 +1,5 @@
 /* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */
 
 // This sample code demonstrates the APIs cs_malloc() & cs_disasm_iter().
 #include <stdio.h>
@@ -34,11 +34,9 @@ static void test()
 #ifdef CAPSTONE_HAS_X86
 #define X86_CODE16 "\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00"
 #define X86_CODE32 "\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00"
-//#define X86_CODE32 "\x0f\xa7\xc0"	// xstorerng
 #define X86_CODE64 "\x55\x48\x8b\x05\xb8\x13\x00\x00"
 #endif
 #ifdef CAPSTONE_HAS_ARM
-//#define ARM_CODE "\x04\xe0\x2d\xe5"
 #define ARM_CODE "\xED\xFF\xFF\xEB\x04\xe0\x2d\xe5\x00\x00\x00\x00\xe0\x83\x22\xe5\xf1\x02\x03\x0e\x00\x00\xa0\xe3\x02\x30\xc1\xe7\x00\x00\x53\xe3"
 #define ARM_CODE2 "\x10\xf1\x10\xe7\x11\xf2\x31\xe7\xdc\xa1\x2e\xf3\xe8\x4e\x62\xf3"
 #define THUMB_CODE "\x70\x47\xeb\x46\x83\xb0\xc9\x68"
@@ -46,24 +44,12 @@ static void test()
 #endif
 #ifdef CAPSTONE_HAS_MIPS
 #define MIPS_CODE "\x0C\x10\x00\x97\x00\x00\x00\x00\x24\x02\x00\x0c\x8f\xa2\x00\x00\x34\x21\x34\x56\x00\x80\x04\x08"
-//#define MIPS_CODE "\x21\x38\x00\x01"
-//#define MIPS_CODE "\x21\x30\xe6\x70"
-//#define MIPS_CODE "\x1c\x00\x40\x14"
 #define MIPS_CODE2 "\x56\x34\x21\x34\xc2\x17\x01\x00"
 #endif
 #ifdef CAPSTONE_HAS_ARM64
-//#define ARM64_CODE "\xe1\x0b\x40\xb9"	// ldr		w1, [sp, #0x8]
-//#define ARM64_CODE "\x00\x40\x21\x4b"	// 	sub		w0, w0, w1, uxtw
-//#define ARM64_CODE "\x21\x7c\x02\x9b"	// mul	x1, x1, x2
-//#define ARM64_CODE "\x20\x74\x0b\xd5"	// dc	zva, x0
-//#define ARM64_CODE "\x20\xfc\x02\x9b"	// mneg	x0, x1, x2
-//#define ARM64_CODE "\x21\x7c\x02\x9b\x21\x7c\x00\x53\x00\x40\x21\x4b\xe1\x0b\x40\xb9\x10\x20\x21\x1e"
-//#define ARM64_CODE "\x21\x7c\x00\x53"
 #define ARM64_CODE "\x09\x00\x38\xd5\xbf\x40\x00\xd5\x0c\x05\x13\xd5\x20\x50\x02\x0e\x20\xe4\x3d\x0f\x00\x18\xa0\x5f\xa2\x00\xae\x9e\x9f\x37\x03\xd5\xbf\x33\x03\xd5\xdf\x3f\x03\xd5\x21\x7c\x02\x9b\x21\x7c\x00\x53\x00\x40\x21\x4b\xe1\x0b\x40\xb9\x20\x04\x81\xda\x20\x08\x02\x8b\x10\x5b\xe8\x3c"
 #endif
-//#define THUMB_CODE "\x0a\xbf" // itet eq
-//#define X86_CODE32 "\x77\x04"	// ja +6
-#ifdef CAPSTONE_HAS_PPC
+#ifdef CAPSTONE_HAS_POWERPC
 #define PPC_CODE "\x80\x20\x00\x00\x80\x3f\x00\x00\x10\x43\x23\x0e\xd0\x44\x00\x80\x4c\x43\x22\x02\x2d\x03\x00\x80\x7c\x43\x20\x14\x7c\x43\x20\x93\x4f\x20\x00\x21\x4c\xc8\x00\x21\x40\x82\x00\x14"
 #endif
 #ifdef CAPSTONE_HAS_SPARC
@@ -168,7 +154,7 @@ static void test()
 			"ARM-64"
 		},
 #endif
-#ifdef CAPSTONE_HAS_PPC
+#ifdef CAPSTONE_HAS_POWERPC
 		{
 			CS_ARCH_PPC,
 			CS_MODE_BIG_ENDIAN,
