@@ -1,5 +1,5 @@
 /* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013 */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -131,6 +131,54 @@ static const char *get_eflag_name(uint64_t flag)
 	}
 }
 
+static const char *get_fpu_flag_name(uint64_t flag)
+{
+	switch (flag) {
+		default:
+			return NULL;
+		case X86_FPU_FLAGS_MODIFY_C0:
+			return "MOD_C0";
+		case X86_FPU_FLAGS_MODIFY_C1:
+			return "MOD_C1";
+		case X86_FPU_FLAGS_MODIFY_C2:
+			return "MOD_C2";
+		case X86_FPU_FLAGS_MODIFY_C3:
+			return "MOD_C3";
+		case X86_FPU_FLAGS_RESET_C0:
+			return "RESET_C0";
+		case X86_FPU_FLAGS_RESET_C1:
+			return "RESET_C1";
+		case X86_FPU_FLAGS_RESET_C2:
+			return "RESET_C2";
+		case X86_FPU_FLAGS_RESET_C3:
+			return "RESET_C3";
+		case X86_FPU_FLAGS_SET_C0:
+			return "SET_C0";
+		case X86_FPU_FLAGS_SET_C1:
+			return "SET_C1";
+		case X86_FPU_FLAGS_SET_C2:
+			return "SET_C2";
+		case X86_FPU_FLAGS_SET_C3:
+			return "SET_C3";
+		case X86_FPU_FLAGS_UNDEFINED_C0:
+			return "UNDEF_C0";
+		case X86_FPU_FLAGS_UNDEFINED_C1:
+			return "UNDEF_C1";
+		case X86_FPU_FLAGS_UNDEFINED_C2:
+			return "UNDEF_C2";
+		case X86_FPU_FLAGS_UNDEFINED_C3:
+			return "UNDEF_C3";
+		case X86_FPU_FLAGS_TEST_C0:
+			return "TEST_C0";
+		case X86_FPU_FLAGS_TEST_C1:
+			return "TEST_C1";
+		case X86_FPU_FLAGS_TEST_C2:
+			return "TEST_C2";
+		case X86_FPU_FLAGS_TEST_C3:
+			return "TEST_C3";
+	}
+}
+
 static void print_insn_detail(csh ud, cs_mode mode, cs_insn *ins)
 {
 	int count, i;
@@ -295,13 +343,27 @@ static void print_insn_detail(csh ud, cs_mode mode, cs_insn *ins)
 		}
 	}
 
-	if (x86->eflags) {
-		printf("\tEFLAGS:");
-		for(i = 0; i <= 45; i++)
-			if (x86->eflags & ((uint64_t)1 << i)) {
-				printf(" %s", get_eflag_name((uint64_t)1 << i));
+	if (x86->eflags || x86->fpu_flags) {
+		for(i = 0; i < ins->detail->groups_count; i++) {
+			if (ins->detail->groups[i] == X86_GRP_FPU) {
+				printf("\tFPU_FLAGS:");
+				for(i = 0; i <= 63; i++)
+					if (x86->fpu_flags & ((uint64_t)1 << i)) {
+						printf(" %s", get_fpu_flag_name((uint64_t)1 << i));
+					}
+				printf("\n");
+				break;
 			}
-		printf("\n");
+		}
+
+		if (i == ins->detail->groups_count) {
+			printf("\tEFLAGS:");
+			for(i = 0; i <= 63; i++)
+				if (x86->eflags & ((uint64_t)1 << i)) {
+					printf(" %s", get_eflag_name((uint64_t)1 << i));
+				}
+			printf("\n");
+		}
 	}
 
 	printf("\n");
@@ -309,32 +371,9 @@ static void print_insn_detail(csh ud, cs_mode mode, cs_insn *ins)
 
 static void test()
 {
-//#define X86_CODE32 "\x01\xd8\x81\xc6\x34\x12\x00\x00\x05\x78\x56\x00\x00"
-//#define X86_CODE32 "\x05\x78\x56\x00\x00"
-//#define X86_CODE32 "\x01\xd8"
-//#define X86_CODE32 "\x05\x23\x01\x00\x00"
-//#define X86_CODE32 "\x8d\x87\x89\x67\x00\x00"
-//#define X86_CODE32 "\xa1\x13\x48\x6d\x3a\x8b\x81\x23\x01\x00\x00\x8b\x84\x39\x23\x01\x00\x00"
-//#define X86_CODE32 "\xb4\xc6"	// mov	ah, 0x6c
-//#define X86_CODE32 "\x77\x04"	// ja +6
 #define X86_CODE64 "\x55\x48\x8b\x05\xb8\x13\x00\x00\xe9\xea\xbe\xad\xde\xff\x25\x23\x01\x00\x00\xe8\xdf\xbe\xad\xde\x74\xff"
-//#define X86_CODE64 "\xe9\x79\xff\xff\xff"	// jmp 0xf7e
-
 #define X86_CODE16 "\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00\x05\x23\x01\x00\x00\x36\x8b\x84\x91\x23\x01\x00\x00\x41\x8d\x84\x39\x89\x67\x00\x00\x8d\x87\x89\x67\x00\x00\xb4\xc6\x66\xe9\xb8\x00\x00\x00\x67\xff\xa0\x23\x01\x00\x00\x66\xe8\xcb\x00\x00\x00\x74\xfc"
 #define X86_CODE32 "\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00\x05\x23\x01\x00\x00\x36\x8b\x84\x91\x23\x01\x00\x00\x41\x8d\x84\x39\x89\x67\x00\x00\x8d\x87\x89\x67\x00\x00\xb4\xc6\xe9\xea\xbe\xad\xde\xff\xa0\x23\x01\x00\x00\xe8\xdf\xbe\xad\xde\x74\xff"
-//#define X86_CODE32 "\x05\x23\x01\x00\x00\x0f\x01\xda"
-//#define X86_CODE32 "\x0f\xa7\xc0"	// xstorerng
-//#define X86_CODE32 "\x64\xa1\x18\x00\x00\x00"	// mov eax, dword ptr fs:[18]
-//#define X86_CODE32 "\x64\xa3\x00\x00\x00\x00"	// mov [fs:0x0], eax
-//#define X86_CODE32 "\xd1\xe1"	// shl ecx, 1
-//#define X86_CODE32 "\xd1\xc8"	// ror eax, 1
-//#define X86_CODE32 "\x83\xC0\x80"	// add	eax, -x80
-//#define X86_CODE32 "\xe8\x26\xfe\xff\xff"		// call	0xe2b
-//#define X86_CODE32 "\xcd\x80"		// int 0x80
-//#define X86_CODE32 "\x24\xb8"		// and    $0xb8,%al
-//#define X86_CODE32 "\xf0\x01\xd8"   // lock add eax,ebx
-//#define X86_CODE32 "\xf3\xaa"		// rep stosb
-//#define X86_CODE32 "\x81\xc6\x23\x01\x00\x00"
 
 	struct platform platforms[] = {
 		{
@@ -396,7 +435,7 @@ static void test()
 			printf("Disasm:\n");
 
 			for (j = 0; j < count; j++) {
-				printf("0x%" PRIx64 ":\t%s\t%s\n\n", insn[j].address, insn[j].mnemonic, insn[j].op_str);
+				printf("0x%" PRIx64 ":\t%s\t%s\n", insn[j].address, insn[j].mnemonic, insn[j].op_str);
 				print_insn_detail(handle, platforms[i].mode, &insn[j]);
 			}
 			printf("0x%" PRIx64 ":\n", insn[j-1].address + insn[j-1].size);
